#!/usr/bin/python
# $Id: IGSCPparser.py,v 1.2 2003/06/15 13:05:12 peter Exp $

""" This is a reference implementation of an IGSCP Parser. It currently
supports the mandatory INFO and the MARK namespace. CONTROL and MOVE are
not yet supported. """


import socket, string, types
from __init__ import *

s = None

#
# An IGSCP error occured. Print a proper message.
#
# Parameters.
#    errorCode - An integer value, see the constants defined in __init__
#    arg       - An optional string containing the wrong command
#
def invalid_IGSCP_command(errorCode, arg=None):
    if errorCode == IGSCP_ERROR_INVALID_NAMESPACE:
        print 'ERROR: Invalid namespace: %s' % arg
    elif errorCode == IGSCP_ERROR_NAMESPACE_NOT_SUPPORTED:
        print 'ERROR: Namespace %s not supported in this client.' % arg
    elif errorCode == IGSCP_ERROR_UNKNOWN_COMMAND:
        print 'ERROR: Unknown command: %s' % arg
    elif errorCode == IGSCP_ERROR_MISSING_COMMAND_PART:
        print 'ERROR: Missing command part after %s.' % arg
    elif errorCode == IGSCP_ERROR_MISSING_ARGUMENTS:
        print 'ERROR: Missing arguments.'
    elif errorCode == IGSCP_ERROR_INVALID_ARGUMENTS:
        print 'ERROR: Invalid arguments: %s' % arg
    else:
        print 'ERROR: Not specified.'

#
# Main method of this module: Parse an IGSCP line
#
def parse_IGSCP(toParse):
    if debug_level >= 1:
        print 'parse_IGSCP:', toParse

    # Cut leading '#' if there is one
    if toParse[0] == '#':
        toParse = toParse[1:len(toParse)]

    # Split command from args
    l = toParse.split(' ')
    command = l[0]
    try:
        # Assemble args in a list or a string if only one is given
        args = l[1:len(l)]
        if len(args) == 0:
            args = None
        elif len(args) == 1:
            args = args[0]
    except IndexError:
        args = None

    if debug_level >= 1:
        print 'command = %s' % command
        print 'args = %s' % args

    l = command.split(':')
    if not len(l):
        print 'Invalid IGSCP command.'  # TODO
        return

    if l[0] == 'INFO':
        parse_IGSCP_Info(l[1:len(l)], args)
    elif l[0] == 'MARK':
        parse_IGSCP_Mark(l[1:len(l)], args)
    elif l[0] == 'CONTROL':
        parse_IGSCP_Control(l[1:len(l)])
    elif l[0] == 'MOVE':
        parse_IGSCP_Move(l[1:len(l)])
    else:
        invalid_IGSCP_command(IGSCP_ERROR_INVALID_NAMESPACE, l[0])


# --------------------------- Namespace INFO ------------------------------

#
# Receiving
#

#
# Parse namespace INFO
#
def parse_IGSCP_Info(command, args):
    if debug_level >= 1:
        print 'parse_IGSCP_Info:', command, args

    if not len(command):
        invalid_IGSCP_command(IGSCP_ERROR_MISSING_COMMAND_PART, IGSCP_NAMESPACE_INFO)
        return
    if command[0] == 'REQUEST':
        if len(command) != 2:
            invalid_IGSCP_command(IGSCP_ERROR_MISSING_COMMAND_PART, command[0])
            return
        parse_IGSCP_Info_Request(command[1])
    elif command[0] == 'VERSION':
        if not args:
            invalid_IGSCP_command(IGSCP_ERROR_MISSING_ARGUMENTS)
            return
        print 'Recieved IGSCP Version %s' % args
    elif command[0] == 'CLIENT':
        if not args:
            invalid_IGSCP_command(IGSCP_ERROR_MISSING_ARGUMENTS)
            return
        print 'Recieved IGSCP Client %s' % args
    elif command[0] == 'FEATURES':
        if not args:
            invalid_IGSCP_command(IGSCP_ERROR_MISSING_ARGUMENTS)
            return
        print 'Recieved IGSCP Features %s' % args
    elif command[0] == 'PHOTO':
        if not args:
            invalid_IGSCP_command(IGSCP_ERROR_MISSING_ARGUMENTS)
            return
        print 'Recieved IGSCP Photo %s' % args
    else:
        invalid_IGSCP_command(IGSCP_ERROR_UNKNOWN_COMMAND, command[0])

#
# Parse namespace INFO:REQUEST
#
def parse_IGSCP_Info_Request(command):
    if debug_level >= 1:
        print 'parse_IGSCP_Info_Request:', command

    if command == 'VERSION':
        send_IGSCP_Version()
    elif command == 'CLIENT':
        send_IGSCP_Client()
    elif command == 'FEATURES':
        send_IGSCP_Features()
    elif command == 'PHOTO':
        send_IGSCP_Photo()
    else:
        invalid_IGSCP_command(IGSCP_ERROR_UNKNOWN_COMMAND, command)

#
# Sending
#

#
# Send INFO:VERSION
#
def send_IGSCP_Version():
    txt = '#INFO:VERSION %s' % IGSCP_VERSION
    print 'Sending (tell): %s' % txt
    if s:
        s.send(txt)

#
# Send INFO:CLIENT
#
def send_IGSCP_Client():
    txt = '#INFO:CLIENT %s' % IGSCP_CLIENT
    print 'Sending (tell): %s' % txt
    if s:
        s.send(txt)

#
# Send INFO:FEATURES
#
def send_IGSCP_Features():
    txt = '#INFO:FEATURES %s' % IGSCP_FEATURES
    print 'Sending (tell): %s' % txt
    if s:
        s.send(txt)

#
# Send INFO:PHOTO
#
def send_IGSCP_Photo():
    txt = '#INFO:PHOTO %s' % 'http://www.myphoto.org'
    print 'Sending (tell): %s' % txt
    if s:
        s.send(txt)


# --------------------------- Namespace MARK ------------------------------

#
# Receiving
#

#
# Parse namespace MARK
#
def parse_IGSCP_Mark(command, args):
    if debug_level >= 1:
        print 'parse_IGSCP_Mark:', command

    if not len(command):
        invalid_IGSCP_command(IGSCP_ERROR_MISSING_COMMAND_PART, IGSCP_NAMESPACE_MARK)
        return
    if len(command) != 1:
        invalid_IGSCP_command(IGSCP_ERROR_UNKNOWN_COMMAND, string.join(command, ':'))
        return
    if command[0] == 'ADD':
        if not args:
            invalid_IGSCP_command(IGSCP_ERROR_MISSING_ARGUMENTS)
            return
        print 'Add mark: %s' % args
    elif command[0] == 'REMOVE':
        if not args:
            invalid_IGSCP_command(IGSCP_ERROR_MISSING_ARGUMENTS)
            return
        if args == 'all':
            print 'Remove all marks'
        else:
            print 'Remove mark: %s' % args
    elif command[0] == 'MOVE':
        if not args:
            invalid_IGSCP_command(IGSCP_ERROR_MISSING_ARGUMENTS)
            return
        elif type(args) != types.ListType or len(args) != 2:
            invalid_IGSCP_command(IGSCP_ERROR_INVALID_ARGUMENTS, args)
            return
        print 'Move mark: %s' % args
    else:
        invalid_IGSCP_command(IGSCP_ERROR_UNKNOWN_COMMAND, command[0])

#
# Sending
#

def send_IGSCP_Add_Mark(x, y, type):
    # txt = '#INFO:MARK:ADD ' + x + y + type
    # print  'Sending (chatter): " + txt);
    pass

##     static void send_Remove_Mark(int x, int y) {
##         String txt = "#INFO:MARK:REMOVE " + x + y + type;
##         System.err.println("Sending (chatter): " + txt);
##         IGSConnection.sendCommand(txt);
##     }
##     static void send_Move_Mark(int x1, int y1, int x2, int y2, int type) {
##         String txt = "#INFO:MARK:REMOVE " + x1 + y1 + x2 + y2 + type;
##         System.err.println("Sending (chatter): " + txt);
##         IGSConnection.sendCommand(txt);
##     }


# --------------------------- Namespace CONTROL ---------------------------

#
# Receiving
#

#
# Parse namespace CONTROL
#
def parse_IGSCP_Control(command):
    if debug_level >= 1:
        print 'parse_IGSCP_Control:', command

    invalid_IGSCP_command(IGSCP_ERROR_NAMESPACE_NOT_SUPPORTED, IGSCP_NAMESPACE_CONTROL)


# --------------------------- Namespace MOVE ------------------------------

#
# Receiving
#

#
# Parse namespace MOVE
#
def parse_IGSCP_Move(command):
    if debug_level >= 1:
        print 'parse_IGSCP_Move:', command

    invalid_IGSCP_command(IGSCP_ERROR_NAMESPACE_NOT_SUPPORTED, IGSCP_NAMESPACE_MOVE)


if __name__ == '__main__':
    import sys, string
    if len(sys.argv) < 2:
        print 'Give an IGSCP command as parameter.'
    else:
        parse_IGSCP(string.join(sys.argv[1:len(sys.argv)], ' '))
