#!/usr/bin/python
# $Id: client.py,v 1.3 2003/06/15 13:03:34 peter Exp $

""" This client module offers a simple client which can be additionally connected
to the server to a real IGS client for easy server control. """


import socket, threading, time, sys, __init__, IGSCP.IGSCPparser, IGSCP.__init__

abort = 0
s = None

#
# Loop for user commands to send simple control strings to the server
#
def input_loop(s):
    while not abort:
        # Get user input
        try:
            data = raw_input('> ')
        except KeyboardInterrupt:  # Catch Ctrl-C
            return
        except EOFError:  # Catch Ctrl-D
            return

        if not data:
            continue

        # Parse client-side commands
        if data[0:2] == 've':
            IGSCP.IGSCPparser.send_IGSCP_Version()
        elif data[0:2] == 'cl':
            IGSCP.IGSCPparser.send_IGSCP_Client()
        elif data[0:2] == 'fe':
            IGSCP.IGSCPparser.send_IGSCP_Features()
        elif data[0:2] == 'ph':
                IGSCP.IGSCPparser.send_IGSCP_Photo()
        elif data[0:2] == 'am':
            IGSCP.IGSCPparser.send_Add_Mark(data[3:len(data)])
        elif data[0:2] == 'rm':
            IGSCP.IGSCPparser.send_Remove_Mark(data[3:len(data)])
        elif data[0:2] == 'mm':
            IGSCP.IGSCPparser.send_Move_Mark(data[3:len(data)])
        else:
            # Send data to server
            s.send(data)
            time.sleep(1)

#
# Establish connection to a server and start input loop for commands
#
def run_client(port=None):
    global abort, s

    if not port:
        port = __init__.PORT

    # Create socket and connect to host:port
    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.connect((__init__.HOST, port))
        print 'Connected to %s %d' % (__init__.HOST, port)
        print 'Use \'exit\' or Ctrl-C and Ctrl-D to quit.\n'
    except:
        print 'Failed to connect to %s %d' % (__init__.HOST, port)
        return

    IGSCP.IGSCPparser.s = s

    t = threading.Thread(target=input_loop, args=(s,))
    t.start()

    # Connection loop
    while 1:
        # Recieve data from server
        try:
            data = s.recv(1024)
        except KeyboardInterrupt:  # Catch Ctrl-C
            break
        except EOFError:  # Catch Ctrl-D
            break
        except socket.error:
            print 'Disconnected.'
            break

        if not data:
            break
        print '->', data

        # Shutdown or disconnected
        if data == 'Shutting down server' or data == 'bye':
            print 'bye bye'
            s.close()
            abort = 1
            sys.exit(0)
        # IGSCP command
        elif data[0] == '#':
            IGSCP.IGSCPparser.parse_IGSCP(data)

    # Shutdown client
    s.close()

#
# Client main method.
# Parse given commandline arguments, connect to server and start the input loop
#
def main(args=None):
    import commandline
    result = None
    if args and len(args) > 1:
        result = commandline.parse_commandline(args[1:len(args)])
        if not result:
            sys.exit(0)

    print 'IGSCPDemo client %s using IGSCP version %s\n' % (__init__.VERSION, IGSCP.__init__.IGSCP_VERSION)
    run_client(result)

if __name__ == '__main__':
    main(sys.argv)
